%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% mspo.pl	
%%
%% genera tots els possibles constraints usant MSPO
%% (es fan algunes simplificacions)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


:-dynamic statuslex/0.

% L=[[L1,R1],...] llista de regles
% LC=[[LCons1,Cami1],[LCons2,Cami2],....]
const_trs([],[[]]):-!.
const_trs(L,LC):-
	const_all_list(L,Landor),
 % Landor=[LOr1,lOr2,...]
 % LOrI=[Or1,Or2,...] ,  Or1=[LCons,Cami]
	dnf(Landor,Lorand),!,
	separar_Or_cami(Lorand,Lorand0),
 % Lorand0=[LAnd1,LAnd2,...]
 % LAnd1=[[And,Cami],....]
	simplifandshead(Lorand0,Lorand1),
	simplif(Lorand1,LC),!.


const_one_trs([],[]):-!.
const_one_trs(L,[AOr3,ACam]):-
	const_all_list(L,Landor),
	dnfone(Landor,A),
	separar_Or_cami1(A,AOr,ACam),
	l_concat(AOr,AOr1),
	simplifand(AOr1,AOr2),
	simplifandshead1(AOr2,AOr3).

const_all_list([],[]):-!.
const_all_list([[N,[S,T]]|L],[OrCam|Landor]):-
	const_all(N,S,T,OrCam),!,
	const_all_list(L,Landor),!.

dnf([O],O):-!.
dnf([O|Landor],Lorand):-
	dnf(Landor,Lorand1),
	mul(O,Lorand1,Lorand).

dnfone([],[]).
dnfone([O|Landor],C):-
	dnfone(Landor,C1),
	mulone(O,C1,C).

mul([],_,[]). 
mul([A|R],Lorand1,Lorand):-
	add(A,Lorand1,Lorand2),
	mul(R,Lorand1,Lorand3),
	concat312(Lorand2,Lorand3,Lorand),!.

mulone([A|_],C1,C):-
	concat312(A,C1,C).
mulone([_|R],C1,C):-
	mulone(R,C1,C).

add(_,[],[]).
add(A,[A1|Lorand1],[A2|Lorand2]):-
	concat312(A,A1,A2),
	add(A,Lorand1,Lorand2),!.


separar_Or_cami([],[]).
separar_Or_cami([AndOrCam|Lorand],[[AndOr,AndLCam]|LorandOut]):-
	separar_Or_cami1(AndOrCam,AndLOr,AndLCam),
	l_concat(AndLOr,AndOr),
	separar_Or_cami(Lorand,LorandOut),!.

separar_Or_cami1([],[],[]):-!.
separar_Or_cami1([Or,Cam|AndOrCam],[Or|AndLOr],[Cam|AndLCam]):-
	separar_Or_cami1(AndOrCam,AndLOr,AndLCam),!.
	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

lex:- assert(statuslex).
nolex:- retractall(statuslex).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

const_all(N,S,T,LS):- 
	statuslex,!,
	findall([C,[N,Cam]],const_gr_sl(S,T,[],C,Cam),L),!,
	check_nonull(L,S,T),
	renvars_list(L,Lc),
	simplif(Lc,LS).

const_all(N,S,T,LS):- 
	findall([C,[N,Cam]],const_gr(S,T,[],C,Cam),L),!,
	check_nonull(L,S,T),
	renvars_list(L,Lc),
	simplif(Lc,LS).


check_nonull([],S,T):-
	renvars([S,T],[S1,T1]),
	miss_no(1,S1,T1),!,fail.
check_nonull(_,_,_).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

const_gr(S,_,_,_,_):-
	var(S),!,fail.
const_gr(S,T,_,_,_):-
	subteq(S,T),!,fail.
const_gr(S,T,C,C,[1,0]):-
	subt(T,S),!.
const_gr(_,T,_,_,_):-
	var(T),!,fail.
const_gr(S,T,_,_,_):-
	llvars(S,[],VS),
	llvars(T,[],VT),
	difer(VT,VS,[_|_]),!,fail.
const_gr(S,T,_,_,_):-
	\+(try_cancel_cycles),
	S =.. [F|LS],
	T =.. [F|LT],
	elim_eq_samepos(LS,LT,LS1,LT1),
	nowf(LS1,LT1),
	leftR0(Def), 
	l_cap_ren(LT,Def,_,_,LTabs),	
	Tabs=..[F|LTabs],
	S=Tabs,!,fail.
const_gr(S,T,C,COut,[3|Cam]):-
	S =.. [_|LS],
	T =.. [_|LT],
	addifnoex(S>=T,C,C1),
	const_gr_mul(LS,LT,C1,COut,Cam).
const_gr(S,T,C,COut,[2|Cam]):-
	T =.. [_|LT],
	addifnoex(S>T,C,C1),
	const_gr_list(S,LT,C1,COut,Cam).
const_gr(S,T,C,C,[1,0]):-
	S =.. [_|LS],
	member212(T,LS),!.	% member22(X,LS),X==T,!.
const_gr(S,T,C,COut,[1,N|Cam]):-
	S =.. [_|LS],
	enumerate(LS,LSen),
	del_repesenum(LSen,LSen1),
	member22([N,X],LSen1),
	const_gr(X,T,C,COut,Cam).

const_gr_list(_,[],L,L,[]):-!.
const_gr_list(S,[T|LT],C,COut,Cam):-
	const_gr(S,T,C,C1,Cam1),
	const_gr_list(S,LT,C1,COut,Cam2),
	concat312(Cam1,Cam2,Cam).

const_gr_mul([],_,_,_,_):-!,fail.
const_gr_mul(_,[],C,C,[]):-!.
const_gr_mul(LS,LT,C,COut,Cam):-
	enumerate(LS,LSen),
	enumerate(LT,LTen),
	difer2enum(LSen,LTen,LSen1,LTen1),!,
	const_gr_mul1(LSen1,LTen1,C,COut,Cam).

const_gr_mul1([],_,_,_,_):-!,fail.
const_gr_mul1(_,[],C,C,[]):-!.
const_gr_mul1(LSen,[[Nt,T]|LTen],C,COut,Cam):-
	member22([Ns,S],LSen),
	const_gr(S,T,C,C1,Cam1),
	const_gr_mul1(LSen,LTen,C1,COut,Cam2),
	concat312([Ns,Nt|Cam1],Cam2,Cam).

%%%
%no ground
difer2enum(Lenum1,Lenum2,L12,L21):- interenum(Lenum1,Lenum2,L),
				diferenum(Lenum1,L,L12),
				diferenum(Lenum2,L,L21),!.

diferenum(L,[],L1):-!,L1=L.
diferenum(L,[X|R],L1):-delete33([_,Y],L,L11),X==Y,!, diferenum(L11,R,L1),!.
diferenum(L,[_|R],L1):-diferenum(L,R,L1),!.


% retorna intersecci de les segones components
interenum(_,[],L):-!,L=[].
interenum([],_,L):-!,L=[].
interenum([[_,X]|L1],L2,[X|L]):-delete33([_,Y],L2,L22),X==Y,!, interenum(L1,L22,L),!.
interenum([_|L1],L2,L):- interenum(L1,L2,L),!.

del_repesenum([],[]):-!.
del_repesenum([[_,X]|R],R1):-
	member22([_,Y],R),X==Y,!,
	del_repesenum(R,R1),!.
del_repesenum([X|R],[X|R1]):-
	del_repesenum(R,R1),!.

%%%

const_ge(S,T,L,L):-S==T.
const_ge(S,T,C,COut):-
	const_gr(S,T,C,COut).


%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%

const_gr_sl(S,_,_,_,_):-
	var(S),!,fail.
const_gr_sl(S,T,_,_,_):-
	subteq(S,T),!,fail.
const_gr_sl(S,T,C,C,[1,0]):-
	subt(T,S),!.
const_gr_sl(_,T,_,_,_):-
	var(T),!,fail.
const_gr_sl(S,T,_,_,_):-
	llvars(S,[],VS),
	llvars(T,[],VT),
	difer(VT,VS,[_|_]),!,fail.
const_gr_sl(S,T,_,_,_):-
	\+(try_cancel_cycles),
	S =.. [F|LS],
	T =.. [F|LT],
	elim_eq_samepos(LS,LT,LS1,LT1),
	nowf(LS1,LT1),
	leftR0(Def), 
	l_cap_ren(LT,Def,_,_,LTabs),	
	Tabs=..[F|LTabs],
	S=Tabs,!,fail.
const_gr_sl(S,T,C,COut,[3|Cam]):-
	S =.. [_|LS],
	T =.. [_|LT],
	addifnoex(S>=T,C,C1),
	const_gr_lex0(LS,LT,LT1,C1,C2,NCam1),
	separar_1_2(LT1,Lnums1,LT11),
	const_gr_sl_list(S,LT11,C2,COut,Cam2),
	concat312(NCam1,[Lnums1],NCam1Nums),
	concat312(NCam1Nums,Cam2,Cam).
const_gr_sl(S,T,C,COut,[2|Cam]):-
	T =.. [_|LT],
	addifnoex(S>T,C,C1),
	const_gr_sl_list(S,LT,C1,COut,Cam).
const_gr_sl(S,T,C,C,[1,0]):-
	S =.. [_|LS],
	member212(T,LS),!.
const_gr_sl(S,T,C,COut,[1,N|Cam]):-
	S =.. [_|LS],
	enumerate(LS,LSen),
	del_repesenum(LSen,LSen1),
	member22([N,X],LSen1),
	const_gr_sl(X,T,C,COut,Cam).

const_gr_sl_list(_,[],L,L,[]):-!.
const_gr_sl_list(S,[T|LT],C,COut,Cam):-
	const_gr_sl(S,T,C,C1,Cam1),
	const_gr_sl_list(S,LT,C1,COut,Cam2),
	concat312(Cam1,Cam2,Cam).

const_gr_lex0(LS,LT,LTRest,C,COut,NCam):-
	enumerate(LS,LSenum),
	enumerate(LT,LTenum),
	elim_eq_samepos(LSenum,LTenum,LS1,LT1),
	const_gr_lex(LS1,LT1,LTRest,C,COut,NCam).

const_gr_lex([],_,_,_,_,_):-!,fail.
const_gr_lex(_,[],[],L,L,[0]):-!.
const_gr_lex([[N,S]|_],[[N,T]|LT],LT,C,COut,[N|Cam]):-
	const_gr_sl(S,T,C,COut,Cam).


const_ge_sl(S,T,C,C):-S==T.
const_ge_sl(S,T,C,COut):-
	const_gr_sl(S,T,C,COut).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elim_eq_samepos([],[],[],[]):-!.
elim_eq_samepos([X|LS],[Y|LT],LS1,LT1):- 
	X==Y,!,
	elim_eq_samepos(LS,LT,LS1,LT1).
elim_eq_samepos([X|LS],[Y|LT],[X|LS1],[Y|LT1]):- 
	elim_eq_samepos(LS,LT,LS1,LT1).


nowf([],[]):-!.
nowf(L,_):- allvars(L),!.
nowf([X],[Y]):-
	var(Y),
	\+subt(Y,X),!.

allvars([]):-!.
allvars([X|R]):- var(X),!, allvars(R),!.


addifnoex(ST,L,L):-
	renvars(ST,ST1),
	member22(C,L),
	renvars(C,C1),
	imply(C1,ST1),!.
addifnoex(ST,L,[ST|L]):-!.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

subteq(X,Y):-X==Y,!.
subteq(_,Y):-var(Y),!,fail.
subteq(X,T):-
	T =.. [_|LT],
	member22(Y,LT),
	subteq(X,Y),!.

subt(X,Y):-X==Y,!,fail.
subt(X,Y):-subteq(X,Y).

simplifand([],[]):- !.
simplifand([O|La],Lf):-
	simplifand1(O,La,L1,Lo),
	simplifand(L1,LR),
	concat312(Lo,LR,Lf),!.

simplifand1(X,[],[],[X]):- !.
simplifand1(X,[Y|L],[Y|L],[]):-
	imply(Y,X),!.
simplifand1(X,[Y|L],L1,Lx):-
	imply(X,Y),!,
	simplifand1(X,L,L1,Lx),!.
simplifand1(X,[Y|L],[Y|L1],Lx):-
	simplifand1(X,L,L1,Lx),!.



%%%%

simplif([],[]):-!.
simplif([X|L],LS):-
	simplif1(X,L,L1,Lx),
	simplif(L1,LR),
	concat312(Lx,LR,LS),!.



simplif1(XC,[],[],[XC]):- !.
simplif1([X,_],[[Y,Cy]|L],[[Y,Cy]|L],[]):-
	imply_list(X,Y),!.
simplif1([X,Cx],[[Y,_]|L],L1,Lx):-
	imply_list(Y,X),!,
	simplif1([X,Cx],L,L1,Lx),!.
simplif1(XC,[YC|L],[YC|L1],Lx):-
	simplif1(XC,L,L1,Lx),!.

imply_list(_,[]):-!.
imply_list(L1,[O|L2]):-
	delete33(O1,L1,L3),
	imply(O1,O),!,
	imply_list(L3,L2),!.

imply(O,O):- !.
imply(S>T,S>=T):- !.


%%%%%%
simplifandshead([],[]).
simplifandshead([[A,Cam]|Lorand],[[A1,Cam]|Lorand1]):-
	simplifand(A,A2),
	simplifandshead1(A2,A1),
	simplifandshead(Lorand,Lorand1),!.

simplifandshead1(A,A1):-
	delete33(X,A,A2),
	(X=(S>T); X=(S>=T)),
	T =..[F|_],
	(X=(S>=T);\+(S =.. [F|_])),
	notheadingleft(F,A2),!,
	simplifandshead1(A2,A1),!.
simplifandshead1(A,A).

notheadingleft(_,[]):-!.
notheadingleft(F,[X|_]):-
	(X=(S>T); X=(S>=T)),
	S =.. [G|_],
	F=G,!,fail.
notheadingleft(F,[_|A]):-
	notheadingleft(F,A).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

renvars_list([],[]).
renvars_list([A|L],[A1|L1]):-
	renvars_list1(A,A1),
	renvars_list(L,L1).

renvars_list1([],[]).
renvars_list1([X|A],[X1|A1]):-
	renvars(X,X1),
	renvars_list1(A,A1).

renvars(X,_):-
	renvars(X,X1,1,_),
	assert(elrenombrado(X1)),
	fail.
renvars(_,X1):-
	retract(elrenombrado(X1)),!.

renvars(X,V,N,N1):- 
	var(X),!,
	give_name0(N,V),!,
	X=V,
	N1 is N+1,!.
renvars(T,T1,N,N1):- !,
	T =.. [F|Args],
	renvars1(Args,Args1,N,N1),
	T1 =.. [F|Args1],!.

renvars1([],[],N,N).
renvars1([S|Args],[S1|Args1],N,N1):-
	renvars(S,S1,N,N2),
	renvars1(Args,Args1,N2,N1),!.


give_name0(N,V):-
	variables([Var|_]),
	name(Var,[95|_]),!,
	give_name_underline(N,V).
give_name0(N,V):-
	give_name(N,V),!.		

give_name(1,'x').
give_name(2,'y'). 
give_name(3,'z').
give_name(N,Var):-
                N>3,
		N1 is N-3,
                ascii( N1, [], L ),
                name( Var, [120|L] ), !.
%88
give_name_underline(1,'_x').
give_name_underline(2,'_y'). 
give_name_underline(3,'_z').
give_name_underline(N,Var):-
                N>3,
		N1 is N-3,
                ascii( N1, [], L ),
                name( Var, [95,120|L] ), !.
%88

ascii( 0, L, L ):-!.
ascii( N, L1, L2 ):-
        K is N mod 10,
        A is K+48,
        N1 is N // 10,
        ascii( N1, [A|L1], L2 ), !.




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Per unions jerrquiques

const_trsM([],_,[[]],_):-!.
const_trsM(L,Symbs,LC,Kind):-
	const_all_list(L,Landor),
 % Landor=[LOr1,lOr2,...]
 % LOrI=[Or1,Or2,...] ,  Or1=[LCons,Cami]
	selec_andor_dnf(Landor,Kind,Lorand),
	separar_Or_cami(Lorand,Lorand0),
 % Lorand0=[LAnd1,LAnd2,...]
 % LAnd1=[[And,Cami],....]
	elim_novalid(Lorand0,Symbs,Lorand1),
	simplify_orand(Lorand1,LC),LC=[_|_].


selec_andor_dnf(Landor,plain,Lorand):-!,
	dnf(Landor,Lorand),!.
selec_andor_dnf(Landor,twosteps,Lorand):-!,
	enumerate(Landor,LandorEn),
	sel_in_or_all(LandorEn,LandorSimpl,LandorRest),
	obtain_altern_dnf(LandorEn,LandorSimpl,LandorRest,Lorand).
selec_andor_dnf(Landor,unaun,[Lorand]):-!,
	dnfone(Landor,Lorand).
selec_andor_dnf(Landor,two_unaun,Lorand):-!,
	enumerate(Landor,LandorEn),
	sel_and_dnf(LandorEn,Lorand).

elim_novalid([],_,[]):-!.
elim_novalid([[Land,Cami]|Lorand],Symbs,[[Land,Cami]|Lorand1]):-
	obt_tops(Land,Sleft,Sright),
	(difer(Sleft,Symbs,[]);difer(Sright,Symbs,[])),!,
	elim_novalid(Lorand,Symbs,Lorand1),!.
elim_novalid([_|Lorand],Symbs,Lorand1):-
	elim_novalid(Lorand,Symbs,Lorand1),!.

obt_tops([],[],[]):-!.
obt_tops(Land,Sleft,Sright):-
	obt_tops1(Land,Sleft0,Sright0),
	del_repes(Sleft0,Sleft),
	del_repes(Sright0,Sright),!.

obt_tops1([],[],[]):-!.
obt_tops1([C|Land],[F|Sleft],[G|Sright]):-
	C=..[_|[S,T]],
	functor(S,F,_),
	functor(T,G,_),
	obt_tops1(Land,Sleft,Sright),!.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Es satisfa dues vegades. La primera retorna nomes una part
% de les alternatives i la segona retorna la resta.
const_trs_2steps([],[[]]):-!.
const_trs_2steps(Rules,LC):-
	const_all_list(Rules,Landor),
	enumerate(Landor,LandorEn),
	sel_in_or_all(LandorEn,LandorSimpl,LandorRest),
 % LandorSimpl,LandorRest=[LOr1,LOr2,...]
 % LOrI=[Or1,Or2,...] ,  Or1=[LCons,Cami]
	obtain_altern_dnf(LandorEn,LandorSimpl,LandorRest,Lorand),
	separar_Or_cami(Lorand,Lorand0),
	simplify_orand(Lorand0,LC).


%%%%%%%%%%%

% LSimpl :  es simplifiquen les alternatives: si n'hi ha que
% segur que no tenen cicle es queda amb aquestes.   
% LRest : conte les alternatives eliminades.
% Les llistes estan enumerades.
sel_in_or_all(Landor,LSimpl,LRest):-
	remove_cycle_inside(Landor,Landor2,Landornocyc,LandorRest),
	concat312(Landornocyc,Landor2,LSimpl),
	elim_null(LandorRest,LRest).
	

elim_null([],[]):-!.
elim_null([[_,[[],_]]|R],R1):-!,
	elim_null(R,R1).
elim_null([X|R],[X|R1]):-!,
	elim_null(R,R1).

% Elimina les alternatives que mirant els > i considerant nomes el top
% generen cicle.
% Si no queda cap alternativa llavors no es simplifica
% Landor2= no simplificats ; (Landornocyc=simplificats ; LandorRest=resta)

remove_cycle_inside([],[],[],[]):-!.
remove_cycle_inside([[I,Or]|Landor1],Landor2,[[I,Or1]|Landornocyc],[[I,OrRest]|LandorRest]):-
	no_cycle_inside(Or,[],Or1),!,
	difer(Or,Or1,OrRest),
	remove_cycle_inside(Landor1,Landor2,Landornocyc,LandorRest).
remove_cycle_inside([IOr|Landor1],[IOr|Landor2],Landornocyc,LandorRest):-
	remove_cycle_inside(Landor1,Landor2,Landornocyc,LandorRest).

no_cycle_inside([],L,L):-!,L=[_|_].
no_cycle_inside([[O,Cam]|Or],L,Lge):-
	nocyclegrin1(O),!,
	no_cycle_inside(Or,[[O,Cam]|L],Lge).
no_cycle_inside([_|Or],L,Lge):-
	no_cycle_inside(Or,L,Lge).
	

nocyclegrin1([]):-!.
nocyclegrin1(LC):-
	obtain_tops_gr(LC,[],LTops),
	check_nocycle(LTops,LC).
	

obtain_tops_gr([],L,L):-!.
obtain_tops_gr([S>T|Rest],L,LOut):-
	S=..[F|_], T=..[G|_],\+(F==G),
	( ( delete33([F,Lg],L,L1),!,
	    addifno(G,Lg,Lg1),
	    L2=[[F,Lg1]|L1] )
	  ;
	  L2=[[F,[G]]|L] ),!,
	obtain_tops_gr(Rest,L2,LOut).
obtain_tops_gr([_>=_|Rest],L,LOut):-
	obtain_tops_gr(Rest,L,LOut).


check_nocycle([],_):-!.
check_nocycle([[F,Lg]|R],L):-
	check_nocycle1(F,Lg,[],L),!,
	check_nocycle(R,L).


check_nocycle1(_,Lg,Ldone,_):-
	difer(Lg,Ldone,[]),!.
check_nocycle1(F,Lg,Ldone,LC):-
	obt_right_Lg(Lg,Lg1,LC),!,
	\+(member22(F,Lg1)),
	sort(Lg1,Lg2),
	union(Lg,Ldone,Ldone2),
	check_nocycle1(F,Lg2,Ldone2,LC),!.


obt_right_Lg([],[],_):-!.
obt_right_Lg([G|Lg],LgOut,LC):-
	findall(H,toprightg(G,H,LC),L1),
	obt_right_Lg(Lg,L2,LC),
	concat312(L1,L2,LgOut),!.

toprightg(G,H,L):-
	member22(C,L), C=..[_|[S,T]], S=..[G|_], T=..[H|_].


%%%%%%%%%%
% Es satisf dues vegades.
% la primera fa dnf dels simplificats
% la segona obt la resta. Si la resta es [] falla
obtain_altern_dnf(_,LandorSimpl,_,Lorand):-
	map(snd2,LandorSimpl,LSimpl), % treu enumeracio
	dnf(LSimpl,Lorand).
obtain_altern_dnf(Landor,_,LandorRest,Lorand):-
	obtain_dnf_rest(Landor,LandorRest,[],Lorand),!,
	Lorand=[_|_].

obtain_dnf_rest(_,[],L,L):-!.
obtain_dnf_rest(Landor,[[I,OrRest]|LRest],L,LOut):-
	delete32([I,_],Landor,Landor1),
	map(snd2,Landor1,Landor2),
	dnf([OrRest|Landor2],Lorand2),
	concat312(L,Lorand2,L1),
	obtain_dnf_rest(Landor,LRest,L1,LOut).

%%%%%%%%%%

simplify_orand(Lorand,LC):-
	simplifandshead(Lorand,Lorand1),
	simplif(Lorand1,LC),!.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% two steps and then one two one.


const_trs_hard1([],[[]]):-!.
const_trs_hard1(Rules,LC):-
  	const_all_list(Rules,Landor),
 % Landor=[LOr1,LOr2,...]
 % LOrI=[Or1,Or2,...] ,  Or1=[LCons,Cami]
	enumerate(Landor,LandorEn),
	sel_and_dnf(LandorEn,Lorand),
	separar_Or_cami(Lorand,Lorand0),
	simplify_orand(Lorand0,LC), LC=[_|_].

sel_and_dnf(Landor,Lorand):-
	remove_cycle_inside(Landor,Landor2,Landornocyc,LRest),
	concat312(Landornocyc,Landor2,LSimpl),
	elim_null(LRest,LRest1),
	obtain_altern_dnf1(Landor,LSimpl,LRest1,Lorand).


obtain_altern_dnf1(_,LandorSimpl,_,Lorand):-
	map(snd2,LandorSimpl,LSimpl), % treu enumeracio
	dnf(LSimpl,Lorand).
obtain_altern_dnf1(Landor,_,LandorRest,Lorand):-
	obtain_dnf_rest1(Landor,LandorRest,Lorand).

obtain_dnf_rest1(Landor,[[I,OrRest]|_],[LOut]):-
	delete32([I,_],Landor,Landor1),
	map(snd2,Landor1,Landor2),
	dnfone([OrRest|Landor2],LOut).
obtain_dnf_rest1(Landor,[_|LRest],LOut):-
	obtain_dnf_rest1(Landor,LRest,LOut).
