%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  checking.pl	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% funcions per comprovar regles correctes i per obtenir
% smbols definits, constructors, variables,...
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% comprova regles sintcticament correctes : 
%       no variables a l'esquerra, 
%	no hi hagi functors que sn variables,
% 	no hi hagi diferents aritats per un smbol.
% Retorna llista de smbols definits i constructors amb 
% la seva aritat
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
check_rules_and_signature(Rules,Def,Const):-
	check_and_defined(Rules,[],Def),
	check_constructors(Rules,Def,[],Const),!.

check_and_defined([],D,D):-!.
check_and_defined([[L,_]|Rules],D,Def):-!,
	check_no_var(L),
	functor(L,F,Ar),
	check_no_varF(F),
	check_and_add([F,Ar],D,D1),
	check_and_defined(Rules,D1,Def),!.
check_and_defined(Rules,_,_):-
	miss_error(0,Rules),!,fail.


check_no_var(L):-
	var(L),
	miss_error(1), !,fail.
check_no_var(_).
check_no_varF(F):-
	var(F),
	miss_error(2), !,fail.
%check_no_varF(F):-
%	(member212(F,[x,y,z]) ; name(F,[120|_])),
%	miss_error(2,F), !,fail.
check_no_varF(_).


check_and_add([F,Ar],D,_):-
	member22([F,Ar1],D),
	\+(Ar=Ar1),
	miss_error(3,F),!,fail.
check_and_add([F,Ar],D,D):-
	member22([F,Ar],D),!.
check_and_add([F,Ar],D,[[F,Ar]|D]).



check_constructors([],_,C,C):-!.
check_constructors([[L,R]|Rules],Def,C,Const):-
	obtain_symbs(L,[],Lsymb),
	obtain_symbs(R,Lsymb,Lsymb1),
	sort(Lsymb1,Lsymb2),
	notwoarities(Lsymb2),
	difer(Lsymb2,Def,Lsymb3),
	difer(Lsymb3,C,Lsymb4),
	concat312(Def,C,Symbcheck),
	check_sign_ok(Lsymb4,Symbcheck),
	concat312(Lsymb4,C,C1),!,
	check_constructors(Rules,Def,C1,Const),!.
		
	
check_sign_ok(Lsymb,Symbcheck):-
	member22([F,Ar],Lsymb),
	member22([F,Ar1],Symbcheck),\+(Ar=Ar1),
	miss_error(3,F),!,fail.

check_sign_ok(_,_):-!.

	

obtain_symbs(Term,Lsymb,Lsymb):-
	var(Term),!.
obtain_symbs(Term,Lsymb,LsymbOut):-
	functor(Term,F,Ar),
	check_no_varF(F),
	Term=..[F|Args], 
	obtain_symbs_list(Args,[[F,Ar]|Lsymb],LsymbOut),!.

obtain_symbs_list([],Lsymb,Lsymb):-!.
obtain_symbs_list([Arg|Rest],Lsymb,LsymbOut):-
	obtain_symbs(Arg,Lsymb,Lsymb1),
	obtain_symbs_list(Rest,Lsymb1,LsymbOut),!.

	
notwoarities(L):-
	delete33([F,_],L,L1),
	member22([F,_],L1),
	miss_error(3,F),!,fail.
notwoarities(_).

%miss_error(0,R):-
%	islist(R),
%	write('Error: The expected format for every rule is [left,right]'),nl.
%miss_error(0,_):-
%	write('Error: A list of rules is expected'),nl.
miss_error(1):-
	write('Syntax Error: The left hand side of a rule is a variable'),nl.
miss_error(2):-
	write('Syntax Error: A functor cannot be a variable'),nl.
miss_error(2,F):-
	write('Syntax Error: symbol '),
	write(F),
	write(' is not permitted'),nl.
miss_error(3,F):-
	write('Syntax Error: symbol '),
	write(F),
	write(' has variable arity'),nl.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% obt les variables (en forma de x,y,z,x1,... que
% tindr el sistema
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
obtain_vars_system(Rules,Vars):-
	findall(N,nvars_regla(Rules,N),LN),
	max_list(LN,M),
	constants(M,Vars),!.


nvars_regla([Rule|_],N):-
	llvars(Rule,[],LV),length21(LV,N).
nvars_regla([_|Rules],N):-
	nvars_regla(Rules,N).


constants(0,[]):-!.
constants(1,[x]):-!.
constants(2,[x,y]):-!.
constants(3,[x,y,z]):-!.
constants(N,[x,y,z|R]):- N>3, afegir_cts(4,N,R).

afegir_cts(N,N,[X]):-!, give_name(N,X).
afegir_cts(I,N,[X|R]):- I<N,give_name(I,X), I1 is I+1, afegir_cts(I1,N,R).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

check_no_name_conflict(_,_,[],[]):-!.
check_no_name_conflict(Def,Const,Vars0,Vars0):-
	unio1(Def,Const,Sign),
	map(head,Sign,Symbols),
	inter(Symbols,Vars0,[]),!.
check_no_name_conflict(_,_,Vars0,Vars):-
	change_name_vars(Vars0,Vars).

change_name_vars([],[]):-!.
change_name_vars([X|R],[Y|R1]):-
	name(X,L),
	name(Y,[95|L]),
	change_name_vars(R,R1),!.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% obt les smbols renombrats del constraint
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
obtain_renamed_tops(Const,Lren):-
	obtain_renamed_tops(Const,[],Lren),!.
	
obtain_renamed_tops([],Lren,Lren):-!.
obtain_renamed_tops([C|Const],Lren,LrenOut):-
	C=..[_|[S,T]],
	functor(S,F,Ar1),
	functor(T,G,Ar2),
	addifno([F,Ar1],Lren,Lren1),
	addifno([G,Ar2],Lren1,Lren2),
	obtain_renamed_tops(Const,Lren2,LrenOut),!.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% comprova que totes les regles satisfan que les
% variables de la dreta apareixen a l'esquerra.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

check_rules_wf(Rules):-
	member22([L,R],Rules),
	renvars([L,R],[L1,R1]),
	rule_no_wf(L1,R1),
	miss_no(1,L1,R1),!,fail.
check_rules_wf(_):-!.

rule_no_wf(L,R):-
	obtain_vars(R,VarsR),	
	contains(L,VarsR),!,fail.
rule_no_wf(_,_):-!.

